from itertools import tee, islice, chain

from wagtail.core.blocks import ListBlock


def previous_and_next(iterable):
    prevs, items, nexts = tee(iterable, 3)
    prevs = chain([None], prevs)
    nexts = chain(islice(nexts, 1, None), [None])
    return zip(prevs, items, nexts)


class GalleryBlock(ListBlock):

    class Meta:
        template = "cast/wagtail_gallery_block.html"

    def add_prev_next(self, gallery):
        for previous_image, current_image, next_image in previous_and_next(gallery):
            current_image.prev = "false" if previous_image is None else f"img-{previous_image.pk}"
            current_image.next = "false" if next_image is None else f"img-{next_image.pk}"

    def get_context(self, gallery, parent_context=None):
        self.add_prev_next(gallery)
        return super().get_context(gallery, parent_context=parent_context)



# Generated by Django 3.1.2 on 2020-10-24 06:13

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailcore', '0052_pagelogentry'),
        ('cast', '0004_homepage_alias_for_page'),
    ]

    operations = [
        migrations.AlterField(
            model_name='homepage',
            name='alias_for_page',
            field=models.ForeignKey(blank=True, default=None, help_text='Make this page an alias for another page, redirecting to it with a non permanent redirect.', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='aliases', to='wagtailcore.page', verbose_name='Redirect to another page'),
        ),
    ]



from itertools import chain

from django.forms import Widget
from django.forms.utils import flatatt
from django.utils.http import urlencode
from django.utils.encoding import force_str
from django.utils.safestring import mark_safe
from django.utils.translation import gettext as _
from django.db.models.fields import BLANK_CHOICE_DASH


class DateFacetWidget(Widget):
    def __init__(self, attrs=None, choices=()):
        super().__init__(attrs)

        self.choices = choices

    def value_from_datadict(self, data, files, name):
        value = super().value_from_datadict(data, files, name)
        self.data = data
        return value

    def render(self, name, value, attrs=None, choices=(), renderer=None):
        if not hasattr(self, "data"):
            self.data = {}
        if value is None:
            value = ""
        final_attrs = self.build_attrs(self.attrs, extra_attrs=attrs)
        output = ["<div%s>" % flatatt(final_attrs)]
        options = self.render_options(choices, [value], name)
        if options:
            output.append(options)
        output.append("</div>")
        return mark_safe("\n".join(output))

    def render_options(self, choices, selected_choices, name):
        selected_choices = set(force_str(v) for v in selected_choices)
        output = []
        for option_value, option_label in chain(self.choices, choices):
            if isinstance(option_label, (list, tuple)):
                for option in option_label:
                    output.append(self.render_option(name, selected_choices, *option))
            else:
                output.append(
                    self.render_option(
                        name, selected_choices, option_value, option_label
                    )
                )
        return "\n".join(output)

    def render_option(self, name, selected_choices, option_value, option_label):
        option_value = force_str(option_value)
        if option_label == BLANK_CHOICE_DASH[0][1]:
            option_label = _("All")
        data = self.data.copy()
        data[name] = option_value
        selected = data == self.data or option_value in selected_choices
        try:
            url = data.urlencode()
        except AttributeError:
            url = urlencode(data)
        return self.option_string() % {
            "attrs": selected and ' class="selected"' or "",
            "query_string": url,
            "label": force_str(option_label),
        }

    def option_string(self):
        return '<div class="cast-date-facet-item"><a%(attrs)s href="?%(query_string)s">%(label)s</a></div>'



from django.conf.urls import url
from django.urls import path, include

from . import views
from . import feeds
from .dashboard_views import DashboardView

app_name = "cast"
urlpatterns = [
    # API
    url(r"^api/", include("cast.api.urls", namespace="api")),
    # Dashboard
    url(r"^dashboard/", view=DashboardView.as_view(), name="dashboard"),
    # Feeds
    path(
        "<slug:slug>/feed/rss.xml",
        view=feeds.LatestEntriesFeed(),
        name="latest_entries_feed",
    ),
    path(
        "<slug:slug>/feed/podcast/<audio_format>/rss.xml",
        view=feeds.RssPodcastFeed(),
        name="podcast_feed_rss",
    ),
    path(
        "<slug:slug>/feed/podcast/<audio_format>/atom.xml",
        view=feeds.AtomPodcastFeed(),
        name="podcast_feed_atom",
    ),
    # Regular django views
    url(
        regex=r"^(?P<slug>[^/]+)/add/$",
        view=views.PostCreateView.as_view(),
        name="post_create",
    ),
    url(
        regex=r"^(?P<blog_slug>[^/]+)/(?P<slug>[^/]+)/update/$",
        view=views.PostUpdateView.as_view(),
        name="post_update",
    ),
    url(
        regex=r"^(?P<blog_slug>[^/]+)/(?P<slug>[^/]+)/$",
        view=views.PostDetailView.as_view(),
        name="post_detail",
    ),
    url(
        regex=r"^(?P<slug>[^/]+)/$",
        view=views.PostsListView.as_view(),
        name="post_list",
    ),
    url(
        regex=r"^(?P<slug>[^/]+)_detail/$",
        view=views.BlogDetailView.as_view(),
        name="blog_detail",
    ),
]



# Generated by Django 3.1.2 on 2020-10-19 14:37

import cast.blocks
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import model_utils.fields
import uuid
import wagtail.core.blocks
import wagtail.core.fields
import wagtail.images.blocks


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('wagtailcore', '0052_pagelogentry'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Audio',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('duration', models.DurationField(blank=True, null=True)),
                ('title', models.CharField(blank=True, max_length=255, null=True)),
                ('subtitle', models.CharField(blank=True, max_length=512, null=True)),
                ('m4a', models.FileField(blank=True, null=True, upload_to='cast_audio/')),
                ('mp3', models.FileField(blank=True, null=True, upload_to='cast_audio/')),
                ('oga', models.FileField(blank=True, null=True, upload_to='cast_audio/')),
                ('opus', models.FileField(blank=True, null=True, upload_to='cast_audio/')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Blog',
            fields=[
                ('page_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='wagtailcore.page')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('author', models.CharField(blank=True, default=None, max_length=255, null=True)),
                ('description', models.CharField(max_length=500)),
                ('uuid', models.UUIDField(default=uuid.uuid4, editable=False)),
                ('email', models.EmailField(blank=True, default=None, max_length=254, null=True)),
                ('comments_enabled', models.BooleanField(default=True, help_text='Whether comments are enabled for this blog.', verbose_name='comments_enabled')),
                ('itunes_categories', models.CharField(blank=True, default='', help_text='A json dict of itunes categories pointing to lists of subcategories. Taken from this list https://validator.w3.org/feed/docs/error/InvalidItunesCategory.html', max_length=512, verbose_name='itunes_categories')),
                ('keywords', models.CharField(blank=True, default='', help_text='A comma-delimitedlist of up to 12 words for iTunes\n            searches. Perhaps include misspellings of the title.', max_length=255, verbose_name='keywords')),
                ('explicit', models.PositiveSmallIntegerField(choices=[(1, 'yes'), (2, 'no'), (3, 'clean')], default=1, help_text='``Clean`` will put the clean iTunes graphic by it.', verbose_name='explicit')),
                ('intro', wagtail.core.fields.RichTextField(blank=True)),
            ],
            options={
                'abstract': False,
            },
            bases=('wagtailcore.page', models.Model),
        ),
        migrations.CreateModel(
            name='Gallery',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='HomePage',
            fields=[
                ('page_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='wagtailcore.page')),
                ('body', wagtail.core.fields.StreamField([('heading', wagtail.core.blocks.CharBlock(classname='full title')), ('paragraph', wagtail.core.blocks.RichTextBlock()), ('image', wagtail.images.blocks.ImageChooserBlock(template='cast/wagtail_image.html')), ('gallery', cast.blocks.GalleryBlock(wagtail.images.blocks.ImageChooserBlock()))])),
            ],
            options={
                'abstract': False,
            },
            bases=('wagtailcore.page',),
        ),
        migrations.CreateModel(
            name='Image',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('original', models.ImageField(height_field='original_height', upload_to='cast_images/originals', width_field='original_width')),
                ('original_height', models.PositiveIntegerField(blank=True, null=True)),
                ('original_width', models.PositiveIntegerField(blank=True, null=True)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='cast_images', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ItunesArtWork',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('original', models.ImageField(height_field='original_height', upload_to='cast_images/itunes_artwork', width_field='original_width')),
                ('original_height', models.PositiveIntegerField(blank=True, null=True)),
                ('original_width', models.PositiveIntegerField(blank=True, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Request',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('ip', models.GenericIPAddressField()),
                ('timestamp', models.DateTimeField()),
                ('status', models.PositiveSmallIntegerField()),
                ('size', models.PositiveIntegerField()),
                ('referer', models.CharField(blank=True, max_length=2048, null=True)),
                ('user_agent', models.CharField(blank=True, max_length=1024, null=True)),
                ('method', models.PositiveSmallIntegerField(choices=[(1, 'GET'), (2, 'HEAD'), (3, 'POST'), (4, 'PUT'), (5, 'PATCH'), (6, 'DELETE'), (7, 'OPTIONS'), (8, 'CONNECT'), (8, 'TRACE')])),
                ('path', models.CharField(max_length=1024)),
                ('protocol', models.PositiveSmallIntegerField(choices=[(1, 'HTTP/1.0'), (2, 'HTTP/1.1'), (3, 'HTTP/2.0')])),
            ],
        ),
        migrations.CreateModel(
            name='Video',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('original', models.FileField(upload_to='cast_videos/')),
                ('poster', models.ImageField(blank=True, null=True, upload_to='cast_videos/poster/')),
                ('poster_seconds', models.FloatField(default=1)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Post',
            fields=[
                ('page_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='wagtailcore.page')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('uuid', models.UUIDField(default=uuid.uuid4, editable=False)),
                ('pub_date', models.DateTimeField(blank=True, null=True)),
                ('visible_date', models.DateTimeField(default=django.utils.timezone.now)),
                ('keywords', models.CharField(blank=True, default='', help_text='A comma-demlimitedlist of up to 12 words for iTunes\n            searches. Perhaps include misspellings of the title.', max_length=255, verbose_name='keywords')),
                ('explicit', models.PositiveSmallIntegerField(choices=[(1, 'yes'), (2, 'no'), (3, 'clean')], default=1, help_text='``Clean`` will put the clean iTunes graphic by it.', verbose_name='explicit')),
                ('block', models.BooleanField(default=False, help_text='Check to block this episode from iTunes because <br />its content might cause the entire show to be <br />removed from iTunes.', verbose_name='block')),
                ('comments_enabled', models.BooleanField(default=True, help_text='Whether comments are enabled for this post.', verbose_name='comments_enabled')),
                ('body', wagtail.core.fields.StreamField([('heading', wagtail.core.blocks.CharBlock(classname='full title')), ('paragraph', wagtail.core.blocks.RichTextBlock()), ('image', wagtail.images.blocks.ImageChooserBlock(template='cast/wagtail_image.html')), ('gallery', cast.blocks.GalleryBlock(wagtail.images.blocks.ImageChooserBlock()))])),
                ('audios', models.ManyToManyField(blank=True, to='cast.Audio')),
                ('galleries', models.ManyToManyField(blank=True, to='cast.Gallery')),
                ('images', models.ManyToManyField(blank=True, to='cast.Image')),
                ('parent_blog', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='cast.blog')),
                ('podcast_audio', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='posts', to='cast.audio')),
                ('videos', models.ManyToManyField(blank=True, to='cast.Video')),
            ],
            options={
                'abstract': False,
            },
            bases=('wagtailcore.page', models.Model),
        ),
        migrations.AddField(
            model_name='gallery',
            name='images',
            field=models.ManyToManyField(to='cast.Image'),
        ),
        migrations.AddField(
            model_name='gallery',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
        migrations.CreateModel(
            name='File',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('original', models.FileField(upload_to='cast_files/')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='blog',
            name='itunes_artwork',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='cast.itunesartwork'),
        ),
        migrations.CreateModel(
            name='ChapterMark',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('start', models.TimeField(verbose_name='Start time of chaptermark')),
                ('title', models.CharField(max_length=255)),
                ('link', models.URLField(blank=True, max_length=2000, null=True)),
                ('image', models.URLField(blank=True, max_length=2000, null=True)),
                ('audio', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='chaptermarks', to='cast.audio')),
            ],
            options={
                'unique_together': {('audio', 'start')},
            },
        ),
    ]



# -*- coding: utf-8
from django.apps import AppConfig
from watson import search as watson


class CastConfig(AppConfig):
    name = "cast"

    def ready(self):
        Post = self.get_model("Post")
        # watson.register(Post)  # use wagtails own search interface FIXME



# Generated by Django 3.1.2 on 2020-10-24 06:07

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailcore', '0052_pagelogentry'),
        ('cast', '0003_remove_post_parent_blog'),
    ]

    operations = [
        migrations.AddField(
            model_name='homepage',
            name='alias_for_page',
            field=models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='aliases', to='wagtailcore.page'),
        ),
    ]



import logging

from django.template import Context
from django.template import Template

from django.shortcuts import get_object_or_404

from .models import Blog

logger = logging.getLogger(__name__)


class RenderPostMixin:
    def render_post(self, post, javascript=True, include_detail=False):
        content = "{}\n{}".format("{% load cast_extras %}", post.content)
        template = Template(content)
        blog_context = Context(
            {"javascript": javascript, "post": post, "include_detail": include_detail}
        )
        blog_context.update(post.media_lookup)
        post.description = template.render(blog_context)


class AddRequestUserMixin:
    user_field_name = "user"

    def form_valid(self, form):
        model = form.save(commit=False)
        setattr(model, self.user_field_name, self.request.user)
        return super().form_valid(form)


class PostChangeMixin:
    def form_valid(self, form):
        post = form.save(commit=False)
        if len(form.cleaned_data["slug"]) == 0:
            post.slug = post.get_slug()
        blog = get_object_or_404(Blog, slug=self.blog_slug)
        post.blog = blog
        return super().form_valid(form)

    def form_invalid(self, form):
        logger.info("form invalid: {}".format(form.errors))
        return super().form_invalid(form)


class GetParamsMixin:
    """Collect all request.GET parameters in a querystring and make them
    available to the template. Needed for pagination:
    href="?page={{ page_obj.next_page_number }}{{ parameters }}"
    """

    initial_params = {}

    def get_other_get_params(self):
        get_copy = self.request.GET.copy()
        get_copy.update(self.initial_params.copy())
        parameters = get_copy.pop("page", True) and get_copy.urlencode()
        if len(parameters) > 0:
            parameters = f"&{parameters}"
        return parameters

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["parameters"] = self.get_other_get_params()
        return context



from django.conf.urls import url

from rest_framework.schemas import get_schema_view

from . import views

app_name = "api"
schema_view = get_schema_view(title="Cast API")

urlpatterns = [
    url(r"^schema/$", schema_view),
    url(r"^$", views.api_root, name="root"),
    # image
    url(r"^images/?$", views.ImageListView.as_view(), name="image_list"),
    url(
        r"^images/(?P<pk>\d+)/?$", views.ImageDetailView.as_view(), name="image_detail"
    ),
    url(
        regex=r"^upload_image/$",
        view=views.ImageCreateView.as_view(),
        name="upload_image",
    ),
    # gallery
    url(r"^gallery/?$", views.GalleryListView.as_view(), name="gallery_list"),
    url(
        r"^gallery/(?P<pk>\d+)/?$",
        views.GalleryDetailView.as_view(),
        name="gallery_detail",
    ),
    # video
    url(r"^videos/?$", views.VideoListView.as_view(), name="video_list"),
    url(
        r"^videos/(?P<pk>\d+)/?$", views.VideoDetailView.as_view(), name="video_detail"
    ),
    url(
        regex=r"^upload_video/$",
        view=views.VideoCreateView.as_view(),
        name="upload_video",
    ),
    # audio
    url(r"^audio/?$", views.AudioListView.as_view(), name="audio_list"),
    url(
        r"^audios/(?P<pk>\d+)/?$", views.AudioDetailView.as_view(), name="audio_detail"
    ),
    url(
        r"^audios/podlove/(?P<pk>\d+)/?$",
        views.AudioPodloveDetailView.as_view(),
        name="audio_podlove_detail",
    ),
    # request
    url(r"^request/?$", views.RequestListView.as_view(), name="request_list"),
]



import logging

from django.db import models
from django.utils import timezone
from django.shortcuts import get_object_or_404
from django.db.models.functions import TruncMonth
from django.contrib.auth.mixins import LoginRequiredMixin
from django.views.generic import ListView, DetailView, CreateView, UpdateView

from django_filters.views import FilterView

from .forms import PostForm
from .filters import PostFilter
from .filters import parse_date_facets
from .models import Blog, Post
from .viewmixins import (
    RenderPostMixin,
    AddRequestUserMixin,
    PostChangeMixin,
    GetParamsMixin,
)

logger = logging.getLogger(__name__)


class BlogsListView(ListView):
    model = Blog
    template_name = "cast/blog_list.html"
    context_object_name = "blogs"


class BlogDetailView(DetailView):
    model = Post
    template_name = "cast/blog_detail.html"
    context_object_name = "blog"


class PostsListView(RenderPostMixin, GetParamsMixin, FilterView):
    model = Post
    template_name = "cast/post_list.html"
    context_object_name = "posts"
    paginate_by = 5
    filterset_class = PostFilter

    def get_queryset(self):
        # self.blog is needed elsewhere
        self.blog = get_object_or_404(Blog, slug=self.kwargs["slug"])
        if not self.request.user.is_authenticated:
            queryset = Post.published.filter(blog=self.blog).order_by("-visible_date")
        else:
            queryset = Post.objects.filter(blog=self.blog).order_by("-created")
        return queryset

    def get_selected_facet(self):
        """Return the currently selected facet. Otherwise we would see
        all date facets that are choosable if no date facet was selected
        because in the final pass over the queryset facet_counts would be
        empty and the selected facet would not be accepted because it's
        not in the fields choices."""
        data = self.request.GET or None
        if data is None:
            return None
        date_facet = data.get("date_facets")
        if date_facet is None or len(date_facet) == 0:
            return None
        return parse_date_facets(date_facet)

    def get_facet_counts(self, filterset_class, kwargs):
        """This does a second pass over the queryset, yes. But facet counts
        are really nice to have, so it's worth it :)."""
        kwargs = {k: v for k, v in kwargs.items()}  # copy kwargs to avoid overwriting

        # get selected facet if set and build the facet counting queryset
        facet_counts = {}
        selected_facet = self.get_selected_facet()
        if selected_facet is not None:
            facet_counts = {"year_month": {selected_facet: 1}}
        kwargs["facet_counts"] = facet_counts
        post_filter = filterset_class(**kwargs)
        facet_queryset = (
            post_filter.qs.order_by()
            .annotate(month=TruncMonth("visible_date"))
            .values("month")
            .annotate(n=models.Count("pk"))
        )

        # build up the date facet counts for final filter pass
        year_month_counts = {}
        for row in facet_queryset:
            year_month_counts[row["month"]] = row["n"]
        return {"year_month": year_month_counts}

    def get_filterset_kwargs(self, filterset_class):
        """Collect kwargs for filterset class. Main point it to gather a list
        of date facets with counts."""
        kwargs = super().get_filterset_kwargs(filterset_class)
        # need to keep self.blog - it's used elsewhere too
        self.blog = get_object_or_404(Blog, slug=self.kwargs["slug"])
        kwargs["blog"] = self.blog
        kwargs["facet_counts"] = self.get_facet_counts(filterset_class, kwargs)
        return kwargs

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["blog"] = self.blog
        for post in context[self.context_object_name]:
            self.render_post(post, include_detail=False)
        return context


class PostDetailView(RenderPostMixin, DetailView):
    model = Post
    template_name = "cast/post_detail.html"
    context_object_name = "post"
    slug_url_kwarg = "slug"
    query_pk_and_slug = True

    def get_queryset(self):
        if not self.request.user.is_authenticated:
            queryset = Post.published.order_by("-pub_date")
        else:
            queryset = Post.objects.order_by("-created")
        return queryset

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        post = context[self.context_object_name]
        self.render_post(post, include_detail=True)
        context["next"] = post.get_absolute_url()
        context["comments_enabled"] = post.comments_are_enabled
        return context


class PostCreateView(
    LoginRequiredMixin, PostChangeMixin, AddRequestUserMixin, CreateView
):
    model = Post
    form_class = PostForm
    template_name = "cast/post_edit.html"
    user_field_name = "author"
    success_msg = "Entry created!"

    def get_initial(self):
        initial = super().get_initial()
        initial["visible_date"] = timezone.now()
        return initial

    def form_valid(self, form):
        self.blog_slug = self.kwargs["slug"]
        return super().form_valid(form)

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        # needed for back button
        context["blog_slug"] = self.kwargs["slug"]
        return context


class PostUpdateView(
    LoginRequiredMixin, PostChangeMixin, AddRequestUserMixin, UpdateView
):
    model = Post
    form_class = PostForm
    template_name = "cast/post_edit.html"
    user_field_name = "author"
    success_msg = "Entry updated!"

    def get_initial_chaptermarks(self):
        chaptermarks = []
        if self.object.podcast_audio is not None:
            for chapter_mark in self.object.podcast_audio.chaptermarks.all():
                chaptermarks.append(chapter_mark.original_line)
        return "\n".join(chaptermarks)

    def get_initial(self):
        initial = super().get_initial()
        initial["is_published"] = self.object.is_published
        initial["chaptermarks"] = self.get_initial_chaptermarks()
        initial["pub_date"] = self.object.pub_date
        return initial

    def form_valid(self, form):
        self.blog_slug = self.kwargs["blog_slug"]
        return super().form_valid(form)

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        # needed for back button
        context["blog_slug"] = self.kwargs["blog_slug"]
        return context



# Generated by Django 3.1.2 on 2020-10-24 05:09

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('cast', '0002_remove_blog_description'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='post',
            name='parent_blog',
        ),
    ]



# Generated by Django 3.1.2 on 2020-10-19 14:44

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('cast', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='blog',
            name='description',
        ),
    ]



from django.core.management.base import BaseCommand
from django.core.files.storage import get_storage_class

from cast.utils import storage_walk_paths


class Command(BaseCommand):
    help = "shows size of media on s3"

    def show_usage(self, paths):
        video_endings = {"mov", "mp4"}
        image_endings = {"jpg", "jpeg", "png"}
        image, video, misc = 0, 0, 0
        for path, size in paths.items():
            ending = path.split(".")[-1].lower()
            if ending in video_endings:
                video += size
            elif ending in image_endings:
                image += size
            else:
                misc += size
        unit = 2 ** 20  # MB
        print("video usage: {}".format(video / unit))
        print("image usage: {}".format(image / unit))
        print("misc  usage: {}".format(misc / unit))
        print("total usage: {}".format(sum(paths.values()) / unit))

    def handle(self, *args, **options):
        s3 = get_storage_class("storages.backends.s3boto3.S3Boto3Storage")()
        paths = {}
        for path in storage_walk_paths(s3):
            size = s3.size(path)
            paths[path] = size
            print(path, size / 2 ** 20)
        self.show_usage(paths)



import json
import pytz  # noqa - will get used soon
import logging

from datetime import datetime, timedelta  # noqa - will get used soon

from django.db.models import Count
from django.views.generic import TemplateView
from django.db.models.functions import TruncWeek
from django.contrib.auth.mixins import LoginRequiredMixin

import plotly.graph_objs as go
from plotly.utils import PlotlyJSONEncoder

from .models import Request

logger = logging.getLogger(__name__)


class DashboardView(LoginRequiredMixin, TemplateView):
    template_name = "cast/dashboard.html"

    def get_day_chart(self):
        qs = (
            Request.objects.extra(select={"day": "date(timestamp)"})
            .values("day")
            .annotate(hits=Count("pk"))
        )
        x, y = [], []
        for num, row in enumerate(qs, 1):
            # trace["x"].append(num)
            x.append(row["day"].strftime("%Y-%m-%d"))
            y.append(row["hits"])
        trace = go.Scatter(x=x, y=y, text="Hits", name="Hits per day")

        layout = go.Layout(
            title=go.layout.Title(text="Hits per day", xref="paper", x=0),
            xaxis=go.layout.XAxis(title=go.layout.xaxis.Title(text="Days")),
            yaxis=go.layout.YAxis(title=go.layout.yaxis.Title(text="Hits")),
        )
        return trace, layout

    def last_day(self, d, day_name):
        days_of_week = [
            "sunday",
            "monday",
            "tuesday",
            "wednesday",
            "thursday",
            "friday",
            "saturday",
        ]
        target_day = days_of_week.index(day_name.lower())
        delta_day = target_day - d.isoweekday()
        if delta_day >= 0:
            delta_day -= 7  # go back 7 days
        return d + timedelta(days=delta_day)

    def get_week_chart(self):
        # last_sunday = self.last_day(pytz.utc.localize(datetime.today()), 'sunday')
        qs = (
            Request.objects
            # .filter(timestamp__lte=last_sunday)
            .annotate(week=TruncWeek("timestamp"))
            .values("week")
            .annotate(hits=Count("pk"))
        )
        x, y = [], []
        for num, row in enumerate(qs, 1):
            # trace["x"].append(num)
            x.append(row["week"].strftime("%Y-%m-%d"))
            y.append(row["hits"])
        trace = go.Scatter(x=x, y=y, text="Hits")

        layout = go.Layout(
            title=go.layout.Title(text="Hits per week", xref="paper", x=0),
            xaxis=go.layout.XAxis(title=go.layout.xaxis.Title(text="Weeks")),
            yaxis=go.layout.YAxis(title=go.layout.yaxis.Title(text="Hits")),
        )
        return trace, layout

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)

        # data/layout for hits per day chart
        day_trace, day_layout = self.get_day_chart()
        day_data = [day_trace]
        context["day_data"] = json.dumps(day_data, cls=PlotlyJSONEncoder)
        context["day_layout"] = json.dumps(day_layout, cls=PlotlyJSONEncoder)

        # data/layout for hits per week chart
        week_trace, week_layout = self.get_week_chart()
        week_data = [week_trace]
        context["week_data"] = json.dumps(week_data, cls=PlotlyJSONEncoder)
        context["week_layout"] = json.dumps(week_layout, cls=PlotlyJSONEncoder)
        return context



import pytz
# import pandas as pd

from io import StringIO
from datetime import datetime

from .models import Request


def get_last_request_position(access_log_path, last_request):
    """
    Find the last position of a request with ip and timestamp in
    an access logfile. Used to find all requests which are not yet immported.
    There could be multiple requests with the same ip and timestamp, therefore
    we have to collect all of them (candidates) and then return just the
    position of the last one.
    """
    if last_request is None:
        return 0
    strptime = datetime.strptime
    last_ip = last_request.ip
    last_timestamp = last_request.timestamp
    candidates = []
    with open(access_log_path) as f:
        for position, line in enumerate(f):
            if last_ip in line:
                date_str = line.split("[")[1].split("]")[0]
                timestamp = strptime(date_str, "%d/%b/%Y:%H:%M:%S %z")
                if timestamp == last_timestamp:
                    candidates.append((position, line))
                if timestamp > last_timestamp:
                    break
    return candidates[-1][0]


def parse_str(x):
    """
    Returns the string delimited by two characters.

    Example:
        `>>> parse_str('[my string]')`
        `'my string'`
    """
    return x[1:-1] if x is not None else x


def access_log_to_buffer(access_log_path, start_position=0, chunk_size=None):
    """
    Read all lines from access_log_path starting at start_position and append
    them to an empty buffer. Return that buffer.
    """
    log_buffer = StringIO()
    with open(access_log_path) as f:
        line_count = 0
        for position, line in enumerate(f):
            if position > start_position:
                log_buffer.write(line)
                line_count += 1
                if chunk_size is not None and line_count == chunk_size:
                    # read only chunk_size lines if set
                    break
    log_buffer.seek(0)
    return log_buffer


def parse_datetime(x):
    """
    Parses datetime with timezone formatted as:
        `[day/month/year:hour:minute:second zone]`

    Example:
        `>>> parse_datetime('13/Nov/2015:11:45:42 +0000')`
        `datetime.datetime(2015, 11, 3, 11, 45, 4, tzinfo=<UTC>)`

    Due to problems parsing the timezone (`%z`) with `datetime.strptime`, the
    timezone will be obtained using the `pytz` library.
    """
    dt = datetime.strptime(x[1:-7], "%d/%b/%Y:%H:%M:%S")
    dt_tz = int(x[-6:-3]) * 60 + int(x[-3:-1])
    return dt.replace(tzinfo=pytz.FixedOffset(dt_tz))


#def get_dataframe_from_position(access_log_path, start_position=0, chunk_size=None):
#    log_buffer = access_log_to_buffer(
#        access_log_path, start_position=start_position, chunk_size=chunk_size
#    )
#    df = pd.read_csv(
#        log_buffer,
#        sep=r'\s(?=(?:[^"]*"[^"]*")*[^"]*$)(?![^\[]*\])',
#        engine="python",
#        na_values="-",
#        header=None,
#        usecols=[0, 3, 4, 5, 6, 7, 8],
#        names=[
#            "ip",
#            "user",
#            "user1",
#            "timestamp",
#            "request",
#            "status",
#            "size",
#            "referer",
#            "user_agent",
#        ],
#        converters={
#            "timestamp": parse_datetime,
#            "request": parse_str,
#            "status": int,
#            "size": int,
#            "referer": parse_str,
#            "user_agent": parse_str,
#        },
#    )
#    df = df.drop(df[df.ip.str.len() > 100].index)
#    try:
#        # breaks on empty df
#        df[["method", "path", "protocol"]] = df.request.str.split(" ", expand=True)
#    except ValueError:
#        pass
#    df = df.drop("request", axis=1)
#    return df


def pandas_rows_to_dict(rows):
    """
    Takes a row from df.iterrows() and makes it json serializable.
    """
    request_method_lookup = {v: k for k, v in Request.REQUEST_METHOD_CHOICES}
    protocol_lookup = {v: k for k, v in Request.HTTP_PROTOCOL_CHOICES}
    # dunno why this only works on single row
    # values = {"referer": None, "user_agent": None}
    # row_dict = row.fillna(value=values).to_dict()
    aux = []
    for row_dict in rows:
        row_dict["method"] = request_method_lookup[row_dict["method"]]
        row_dict["protocol"] = protocol_lookup[row_dict["protocol"]]
        row_dict["timestamp"] = row_dict["timestamp"].isoformat()
        for attr in ("method", "protocol", "status", "size"):
            row_dict[attr] = str(row_dict[attr])
        aux.append(row_dict)
    return aux



import os
import requests

from getpass import getpass

from django.core.management.base import BaseCommand


class Command(BaseCommand):
    help = "Get api token for user providing username/password"

    def handle(self, *args, **options):
        username = os.environ.get("USERNAME", "analytics")
        obtain_token_url = os.environ.get("OBTAIN_TOKEN_URL")

        params = {"username": username, "password": getpass()}
        r = requests.post(obtain_token_url, data=params)
        token = r.json()["token"]
        print("token: ", token)



import os
import time
import requests

import dateutil.parser
from pathlib import Path
from collections import namedtuple

from django.core.management.base import BaseCommand

from ...access_log import pandas_rows_to_dict
from ...access_log import get_last_request_position
from ...access_log import get_dataframe_from_position


def insert_log_chunk(request_api_url, api_token, access_log_path, chunk_size=1000):
    now = time.time()
    last_request = None
    headers = {"Authorization": f"Token {api_token}"}
    result = requests.get(request_api_url, headers=headers)
    try:
        last_request_data = result.json()["results"][0]
        last_request_data["timestamp"] = dateutil.parser.parse(
            last_request_data["timestamp"]
        )
        last_request = namedtuple("Request", last_request_data.keys())(
            *last_request_data.values()
        )
    except IndexError:
        pass
    print("last_request: ", last_request)
    last_position = get_last_request_position(access_log_path, last_request)
    print("last_position: ", last_position)
    df = get_dataframe_from_position(
        access_log_path, start_position=last_position, chunk_size=chunk_size
    )
    if df.shape[0] == 0:
        # no more lines
        return None, True
    print("get df: ", time.time() - now)
    raw_rows = df.iloc[:chunk_size].fillna("").to_dict(orient="rows")
    rows = pandas_rows_to_dict(raw_rows)
    print("transform rows: ", time.time() - now)
    result = requests.post(request_api_url, json=rows, headers=headers)
    print("total chunk: ", time.time() - now)
    return result, False


class Command(BaseCommand):
    help = "Import requests from an access.log file."

    def handle(self, *args, **options):
        request_api_url = os.environ.get("REQUEST_API_URL")
        access_log_path = Path(os.environ.get("ACCESS_LOG_PATH"))
        api_token = os.environ.get("API_TOKEN")
        print(request_api_url, access_log_path, api_token)
        now = time.time()
        done = False
        while not done:
            result, done = insert_log_chunk(
                request_api_url, api_token, access_log_path, chunk_size=20000
            )
        print("total: ", time.time() - now)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['cast',
 'cast.api',
 'cast.management.commands',
 'cast.migrations',
 'cast.templatetags']

package_data = \
{'': ['*'],
 'cast': ['locale/de/LC_MESSAGES/*',
          'management/*',
          'static/css/*',
          'static/img/*',
          'static/img/cast/*',
          'static/js/*',
          'static/js/cast/*',
          'static/js/cast/web-player/*',
          'templates/*',
          'templates/cast/base.html',
          'templates/cast/base.html',
          'templates/cast/base.html',
          'templates/cast/base.html',
          'templates/cast/base.html',
          'templates/cast/base.html',
          'templates/cast/base.html',
          'templates/cast/base.html',
          'templates/cast/base.html',
          'templates/cast/base.html',
          'templates/cast/base.html',
          'templates/cast/base.html',
          'templates/cast/base.html',
          'templates/cast/base.html',
          'templates/cast/base.html',
          'templates/cast/base.html',
          'templates/cast/blog_detail.html',
          'templates/cast/blog_detail.html',
          'templates/cast/blog_detail.html',
          'templates/cast/blog_detail.html',
          'templates/cast/blog_detail.html',
          'templates/cast/blog_detail.html',
          'templates/cast/blog_detail.html',
          'templates/cast/blog_detail.html',
          'templates/cast/blog_detail.html',
          'templates/cast/blog_detail.html',
          'templates/cast/blog_detail.html',
          'templates/cast/blog_detail.html',
          'templates/cast/blog_detail.html',
          'templates/cast/blog_detail.html',
          'templates/cast/blog_detail.html',
          'templates/cast/blog_detail.html',
          'templates/cast/blog_index_page.html',
          'templates/cast/blog_index_page.html',
          'templates/cast/blog_index_page.html',
          'templates/cast/blog_index_page.html',
          'templates/cast/blog_index_page.html',
          'templates/cast/blog_index_page.html',
          'templates/cast/blog_index_page.html',
          'templates/cast/blog_index_page.html',
          'templates/cast/blog_index_page.html',
          'templates/cast/blog_index_page.html',
          'templates/cast/blog_index_page.html',
          'templates/cast/blog_index_page.html',
          'templates/cast/blog_index_page.html',
          'templates/cast/blog_index_page.html',
          'templates/cast/blog_index_page.html',
          'templates/cast/blog_index_page.html',
          'templates/cast/blog_list_of_posts.html',
          'templates/cast/blog_list_of_posts.html',
          'templates/cast/blog_list_of_posts.html',
          'templates/cast/blog_list_of_posts.html',
          'templates/cast/blog_list_of_posts.html',
          'templates/cast/blog_list_of_posts.html',
          'templates/cast/blog_list_of_posts.html',
          'templates/cast/blog_list_of_posts.html',
          'templates/cast/blog_list_of_posts.html',
          'templates/cast/blog_list_of_posts.html',
          'templates/cast/blog_list_of_posts.html',
          'templates/cast/blog_list_of_posts.html',
          'templates/cast/blog_list_of_posts.html',
          'templates/cast/blog_list_of_posts.html',
          'templates/cast/blog_list_of_posts.html',
          'templates/cast/blog_list_of_posts.html',
          'templates/cast/blog_page.html',
          'templates/cast/blog_page.html',
          'templates/cast/blog_page.html',
          'templates/cast/blog_page.html',
          'templates/cast/blog_page.html',
          'templates/cast/blog_page.html',
          'templates/cast/blog_page.html',
          'templates/cast/blog_page.html',
          'templates/cast/blog_page.html',
          'templates/cast/blog_page.html',
          'templates/cast/blog_page.html',
          'templates/cast/blog_page.html',
          'templates/cast/blog_page.html',
          'templates/cast/blog_page.html',
          'templates/cast/blog_page.html',
          'templates/cast/blog_page.html',
          'templates/cast/cast_base.html',
          'templates/cast/cast_base.html',
          'templates/cast/cast_base.html',
          'templates/cast/cast_base.html',
          'templates/cast/cast_base.html',
          'templates/cast/cast_base.html',
          'templates/cast/cast_base.html',
          'templates/cast/cast_base.html',
          'templates/cast/cast_base.html',
          'templates/cast/cast_base.html',
          'templates/cast/cast_base.html',
          'templates/cast/cast_base.html',
          'templates/cast/cast_base.html',
          'templates/cast/cast_base.html',
          'templates/cast/cast_base.html',
          'templates/cast/cast_base.html',
          'templates/cast/dashboard.html',
          'templates/cast/dashboard.html',
          'templates/cast/dashboard.html',
          'templates/cast/dashboard.html',
          'templates/cast/dashboard.html',
          'templates/cast/dashboard.html',
          'templates/cast/dashboard.html',
          'templates/cast/dashboard.html',
          'templates/cast/dashboard.html',
          'templates/cast/dashboard.html',
          'templates/cast/dashboard.html',
          'templates/cast/dashboard.html',
          'templates/cast/dashboard.html',
          'templates/cast/dashboard.html',
          'templates/cast/dashboard.html',
          'templates/cast/dashboard.html',
          'templates/cast/home_page.html',
          'templates/cast/home_page.html',
          'templates/cast/home_page.html',
          'templates/cast/home_page.html',
          'templates/cast/home_page.html',
          'templates/cast/home_page.html',
          'templates/cast/home_page.html',
          'templates/cast/home_page.html',
          'templates/cast/home_page.html',
          'templates/cast/home_page.html',
          'templates/cast/home_page.html',
          'templates/cast/home_page.html',
          'templates/cast/home_page.html',
          'templates/cast/home_page.html',
          'templates/cast/home_page.html',
          'templates/cast/home_page.html',
          'templates/cast/image_form.html',
          'templates/cast/image_form.html',
          'templates/cast/image_form.html',
          'templates/cast/image_form.html',
          'templates/cast/image_form.html',
          'templates/cast/image_form.html',
          'templates/cast/image_form.html',
          'templates/cast/image_form.html',
          'templates/cast/image_form.html',
          'templates/cast/image_form.html',
          'templates/cast/image_form.html',
          'templates/cast/image_form.html',
          'templates/cast/image_form.html',
          'templates/cast/image_form.html',
          'templates/cast/image_form.html',
          'templates/cast/image_form.html',
          'templates/cast/post.html',
          'templates/cast/post.html',
          'templates/cast/post.html',
          'templates/cast/post.html',
          'templates/cast/post.html',
          'templates/cast/post.html',
          'templates/cast/post.html',
          'templates/cast/post.html',
          'templates/cast/post.html',
          'templates/cast/post.html',
          'templates/cast/post.html',
          'templates/cast/post.html',
          'templates/cast/post.html',
          'templates/cast/post.html',
          'templates/cast/post.html',
          'templates/cast/post.html',
          'templates/cast/post_detail.html',
          'templates/cast/post_detail.html',
          'templates/cast/post_detail.html',
          'templates/cast/post_detail.html',
          'templates/cast/post_detail.html',
          'templates/cast/post_detail.html',
          'templates/cast/post_detail.html',
          'templates/cast/post_detail.html',
          'templates/cast/post_detail.html',
          'templates/cast/post_detail.html',
          'templates/cast/post_detail.html',
          'templates/cast/post_detail.html',
          'templates/cast/post_detail.html',
          'templates/cast/post_detail.html',
          'templates/cast/post_detail.html',
          'templates/cast/post_detail.html',
          'templates/cast/post_edit.html',
          'templates/cast/post_edit.html',
          'templates/cast/post_edit.html',
          'templates/cast/post_edit.html',
          'templates/cast/post_edit.html',
          'templates/cast/post_edit.html',
          'templates/cast/post_edit.html',
          'templates/cast/post_edit.html',
          'templates/cast/post_edit.html',
          'templates/cast/post_edit.html',
          'templates/cast/post_edit.html',
          'templates/cast/post_edit.html',
          'templates/cast/post_edit.html',
          'templates/cast/post_edit.html',
          'templates/cast/post_edit.html',
          'templates/cast/post_edit.html',
          'templates/cast/post_list.html',
          'templates/cast/post_list.html',
          'templates/cast/post_list.html',
          'templates/cast/post_list.html',
          'templates/cast/post_list.html',
          'templates/cast/post_list.html',
          'templates/cast/post_list.html',
          'templates/cast/post_list.html',
          'templates/cast/post_list.html',
          'templates/cast/post_list.html',
          'templates/cast/post_list.html',
          'templates/cast/post_list.html',
          'templates/cast/post_list.html',
          'templates/cast/post_list.html',
          'templates/cast/post_list.html',
          'templates/cast/post_list.html',
          'templates/cast/video_form.html',
          'templates/cast/video_form.html',
          'templates/cast/video_form.html',
          'templates/cast/video_form.html',
          'templates/cast/video_form.html',
          'templates/cast/video_form.html',
          'templates/cast/video_form.html',
          'templates/cast/video_form.html',
          'templates/cast/video_form.html',
          'templates/cast/video_form.html',
          'templates/cast/video_form.html',
          'templates/cast/video_form.html',
          'templates/cast/video_form.html',
          'templates/cast/video_form.html',
          'templates/cast/video_form.html',
          'templates/cast/video_form.html',
          'templates/cast/wagtail_gallery_block.html',
          'templates/cast/wagtail_gallery_block.html',
          'templates/cast/wagtail_gallery_block.html',
          'templates/cast/wagtail_gallery_block.html',
          'templates/cast/wagtail_gallery_block.html',
          'templates/cast/wagtail_gallery_block.html',
          'templates/cast/wagtail_gallery_block.html',
          'templates/cast/wagtail_gallery_block.html',
          'templates/cast/wagtail_gallery_block.html',
          'templates/cast/wagtail_gallery_block.html',
          'templates/cast/wagtail_gallery_block.html',
          'templates/cast/wagtail_gallery_block.html',
          'templates/cast/wagtail_gallery_block.html',
          'templates/cast/wagtail_gallery_block.html',
          'templates/cast/wagtail_gallery_block.html',
          'templates/cast/wagtail_gallery_block.html',
          'templates/cast/wagtail_image.html',
          'templates/cast/wagtail_image.html',
          'templates/cast/wagtail_image.html',
          'templates/cast/wagtail_image.html',
          'templates/cast/wagtail_image.html',
          'templates/cast/wagtail_image.html',
          'templates/cast/wagtail_image.html',
          'templates/cast/wagtail_image.html',
          'templates/cast/wagtail_image.html',
          'templates/cast/wagtail_image.html',
          'templates/cast/wagtail_image.html',
          'templates/cast/wagtail_image.html',
          'templates/cast/wagtail_image.html',
          'templates/cast/wagtail_image.html',
          'templates/cast/wagtail_image.html',
          'templates/cast/wagtail_image.html',
          'templates/comments/*']}

install_requires = \
['Pillow>=7,<8',
 'coreapi>=2.3.3,<3.0.0',
 'django-allauth>=0.42,<0.43',
 'django-ckeditor>=6,<7',
 'django-environ>=0.4.5,<0.5.0',
 'django-filepond>=0.1.2,<0.2.0',
 'django-filter>=2.2.0,<3.0.0',
 'django-imagekit>=4.0.2,<5.0.0',
 'django-model-utils>=4.0.0,<5.0.0',
 'django-threadedcomments>=1.2,<2.0',
 'django-watson>=1.5.4,<2.0.0',
 'django>=3.1,<4.0',
 'djangorestframework>=3.11.0,<4.0.0',
 'feedparser>=6,<7',
 'markdown>=3.2.1,<4.0.0',
 'plotly>=4.5.2,<5.0.0',
 'python-slugify>=4.0.0,<5.0.0',
 'wagtail>=2.8,<3.0',
 'wagtail_srcset>=0.1,<0.2']

entry_points = \
{'console_scripts': ['autoformat = commands:black',
                     'clean = commands:clean',
                     'clean-build = commands:clean_build',
                     'clean-pyc = commands:clean_pyc',
                     'docs = commands:docs',
                     'lint = commands:flake8',
                     'notebook = commands:notebook',
                     'show_coverage = commands:coverage',
                     'test = commands:test']}

setup_kwargs = {
    'name': 'django-cast',
    'version': '0.1.33',
    'description': 'Just another blogging / podcasting package',
    'long_description': '',
    'author': 'Jochen Wersdörfer',
    'author_email': 'jochen@wersdoerfer.de',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/ephes/django-cast',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



import logging

from rest_framework import serializers

from ..models import Image, Video, Gallery, Audio, Request

logger = logging.getLogger(__name__)


class ImageSerializer(serializers.HyperlinkedModelSerializer):
    url = serializers.HyperlinkedIdentityField(view_name="cast:api:image_detail")
    srcset = serializers.ReadOnlyField()
    thumbnail_src = serializers.ReadOnlyField()
    full_src = serializers.ReadOnlyField()

    class Meta:
        model = Image
        fields = ("id", "url", "original", "srcset", "thumbnail_src", "full_src")


class VideoSerializer(serializers.HyperlinkedModelSerializer):
    url = serializers.HyperlinkedIdentityField(view_name="cast:api:video_detail")
    poster = serializers.ImageField(read_only=True, allow_empty_file=True)
    poster_thumbnail = serializers.ImageField(read_only=True, allow_empty_file=True)

    class Meta:
        model = Video
        fields = ("id", "url", "original", "poster", "poster_thumbnail")


class AudioSerializer(serializers.HyperlinkedModelSerializer):
    url = serializers.HyperlinkedIdentityField(view_name="cast:api:audio_detail")
    podlove = serializers.HyperlinkedIdentityField(
        view_name="cast:api:audio_podlove_detail"
    )

    class Meta:
        model = Audio
        fields = ("id", "name", "file_formats", "url", "podlove", "mp3")


class AudioPodloveSerializer(serializers.HyperlinkedModelSerializer):
    audio = serializers.ListField()
    chapters = serializers.ListField()
    duration = serializers.CharField(source="duration_str")

    class Meta:
        model = Audio
        fields = ("title", "subtitle", "audio", "duration", "chapters")


class GallerySerializer(serializers.HyperlinkedModelSerializer):
    url = serializers.HyperlinkedIdentityField(view_name="cast:api:gallery_detail")
    images = serializers.PrimaryKeyRelatedField(many=True, queryset=Image.objects.all())

    def create(self, validated_data):
        user = self.context["request"].user
        gallery = Gallery.objects.create(user=user)
        for image in validated_data["images"]:
            gallery.images.add(image)
        return gallery

    class Meta:
        model = Gallery
        fields = ("id", "url", "images")


class RequestSerializer(serializers.ModelSerializer):
    class Meta:
        model = Request
        fields = "__all__"
